//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
// Parameters file for the SD card controller

// States of SD controller FSM
`define SD_IDLE         4'b0000             // Idle state
`define SD_SRR          4'b0001             // Single Register Read
`define SD_SRR_WB       4'b0010             // Transmit to WB cntr received byte 
`define SD_SRW          4'b0011             // Single Register Write			
`define SD_MRR          4'b0100             // Multiple registers read 
`define SD_MRR_WB       4'b0101             // Transmit to WB cntr received data
`define SD_MRW          4'b0110             // Multiple registers write
`define SD_RDB          4'b0111             // Read data buffer
`define SD_RDB_WB       4'b1000             // Transmit read word over WB
`define SD_RDB_ER       4'b1001             // Error occured during buffer reception
`define SD_WDB          4'b1010             // Read data from WB buffer
`define SD_WDB_WM       4'b1011             // Write dword read from SD_WDB state to the flash memory
`define SD_WDB_WB       4'b1100             // Write back over WB status of write buffer operation

// Commands to SD controller FSM
`define CMD_IDLE        4'b0000             // Idle
`define CMD_SRR         4'b0001             // Command to read Single Register						  
`define CMD_SRW         4'b0010             // Command to write Single Register
`define CMD_MRR         4'b0011             // Command to read Multiple Registers (4)
`define CMD_MRW         4'b0100             // Command to write Multiple Registers (4)
`define CMD_RDB         4'b0101             // Read data buffer
`define CMD_WDB         4'b0110             // Write data buffer

// States of SD Wishbone controller FSM
`define WBSD_IDLE       4'b0000             // Idle state
`define WBSD_BREAD      4'b0001             // Bus read
`define WBSD_EXWRITE    4'b0010             // Bus external write
`define WBSD_BWRITE     4'b0011             // Bus write
`define WBSD_DECODE     4'b0100             // Decode command
`define WBSD_WREAD      4'b0101             // To read one additional word for CMD_MRW command
`define WBSD_WREAD_T    4'b0110             // Additional word for CMD_MRW has been read
`define WBSD_BUF_READ   4'b0111             // State to read data buffer

// Various parameters
`define WBSD_BASE       32'hFFFF0000        // Base address of SD controller in WB address space
`define BUF_S           7'h40               // Start address of the data buffer
`define ER_PATTERN      32'hFEFEFEFE        // Pattern transmitted over WB when error occured
`define TO_PATTERN      32'h00000001        // Pattern transmitted over WB if Timeout occured
`define C_PATTERN       32'h00000002        // Pattern transmitted over WB if transmittion successfully completed
`define TO_VALUE        16'h0FFF            // Timeout value for SD_WDB_WM state	 
`define TO_RDB          16'h0FFF            // Timeout value for SD_RDB state
