//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
// FSM for SD controller
//
`include "sd_parms.v"

module sdcard_fsm (
  clk_i, rst_i, cmd_i, bus_cycle_done_i, next_state1_i,
  buf_er_i, next_state2_i,
  sd_fsm_o
  );
  input clk_i;              // Input clock signal
  input rst_i;              // Input reset signal
  input [3:0] cmd_i;        // Input command for the FSM
  input bus_cycle_done_i;   // Bus cycle completion signal
  input next_state1_i;      // Signal to switch FSM to the next state
  input buf_er_i;           // Signal that error occured during buffer operations
  input next_state2_i;      // Second signal to switch FSM to the next stage
  output [3:0] sd_fsm_o;    // Output FSM state

// Internal registers and wires
  reg [3:0] sd_fsm_state;
  reg [3:0] sd_fsm_next_state;
  
  assign sd_fsm_o = sd_fsm_state;
  
  always @ (sd_fsm_state or cmd_i or bus_cycle_done_i or next_state1_i
            or buf_er_i or next_state2_i)
    begin
      sd_fsm_next_state = `SD_IDLE;
      case (sd_fsm_state)       // Synthesis full case
        `SD_IDLE:
          begin
            case (cmd_i)        // Synthesis full case
              `CMD_SRR: sd_fsm_next_state = `SD_SRR;	
              `CMD_SRW: sd_fsm_next_state = `SD_SRW;
              `CMD_MRR: sd_fsm_next_state = `SD_MRR;
              `CMD_MRW: sd_fsm_next_state = `SD_MRW;
              `CMD_RDB: sd_fsm_next_state = `SD_RDB;
              `CMD_WDB: sd_fsm_next_state = `SD_WDB;
              default: sd_fsm_next_state = `SD_IDLE;
            endcase
          end
        `SD_SRR:
          begin
            if (bus_cycle_done_i)
              sd_fsm_next_state = `SD_SRR_WB;
            else
              sd_fsm_next_state = `SD_SRR;
          end
        `SD_SRR_WB:
          begin
            if (next_state1_i)
              sd_fsm_next_state = `SD_IDLE;
            else
              sd_fsm_next_state = `SD_SRR_WB;
          end										
        `SD_SRW:
          begin
            if (bus_cycle_done_i)
              sd_fsm_next_state = `SD_IDLE;
            else
             sd_fsm_next_state = `SD_SRW;
          end										
        `SD_MRR:
          begin
            if (bus_cycle_done_i)
              sd_fsm_next_state = `SD_MRR_WB;
            else
              sd_fsm_next_state = `SD_MRR;
          end							 
        `SD_MRR_WB:
          begin
            if (next_state1_i)
              sd_fsm_next_state = `SD_IDLE;
            else
              sd_fsm_next_state = `SD_MRR_WB;
          end										
        `SD_MRW:
          begin
            if (bus_cycle_done_i)
              sd_fsm_next_state = `SD_IDLE;
            else
              sd_fsm_next_state = `SD_MRW;
          end
        `SD_RDB:
          begin
            if (buf_er_i)
              sd_fsm_next_state = `SD_RDB_ER;
            else if (bus_cycle_done_i)
              sd_fsm_next_state = `SD_RDB_WB;
            else
              sd_fsm_next_state = `SD_RDB;
          end
        `SD_RDB_WB:
          begin
            if (next_state1_i)
              sd_fsm_next_state = `SD_IDLE;
            else if (next_state2_i)
              sd_fsm_next_state = `SD_RDB;
            else
              sd_fsm_next_state = `SD_RDB_WB;
          end
        `SD_RDB_ER:
          begin
            if (next_state2_i)
              sd_fsm_next_state = `SD_IDLE;
            else  
              sd_fsm_next_state = `SD_RDB_ER;
          end
        `SD_WDB:
          begin
            if (next_state1_i)
              sd_fsm_next_state = `SD_WDB_WM;
            else if (next_state2_i)
              sd_fsm_next_state = `SD_WDB_WB;
            else
              sd_fsm_next_state = `SD_WDB;
          end
        `SD_WDB_WM:
          begin
            if (~next_state1_i && ~next_state2_i)
              sd_fsm_next_state = `SD_WDB;
            else
              sd_fsm_next_state = `SD_WDB_WM;
          end
        `SD_WDB_WB:
          begin
            if (~next_state2_i)
              sd_fsm_next_state = `SD_IDLE;
            else
              sd_fsm_next_state = `SD_WDB_WB;
          end
      endcase
    end  
  
  always @ (negedge clk_i)
    begin
      if (!rst_i)
        begin
          sd_fsm_state <= `SD_IDLE;
        end
      else
        begin
          sd_fsm_state <= sd_fsm_next_state;
        end
    end
endmodule